// Generated by CoffeeScript 1.12.7
var feedback, scoring;

scoring = require('./scoring');

feedback = {
  default_feedback: {
    warning: '',
    suggestions: ["用几句话，避免常用短语", "不需要符号，数字或大写字母"]
  },
  get_feedback: function(score, sequence) {
    var extra_feedback, i, len, longest_match, match, ref;
    if (sequence.length === 0) {
      return this.default_feedback;
    }
    if (score > 2) {
      return {
        warning: '',
        suggestions: []
      };
    }
    longest_match = sequence[0];
    ref = sequence.slice(1);
    for (i = 0, len = ref.length; i < len; i++) {
      match = ref[i];
      if (match.token.length > longest_match.token.length) {
        longest_match = match;
      }
    }
    feedback = this.get_match_feedback(longest_match, sequence.length === 1);
    extra_feedback = '添加1个或2个单词，最好是不常用的单词';
    if (feedback != null) {
      feedback.suggestions.unshift(extra_feedback);
      if (feedback.warning == null) {
        feedback.warning = '';
      }
    } else {
      feedback = {
        warning: '',
        suggestions: [extra_feedback]
      };
    }
    return feedback;
  },
  get_match_feedback: function(match, is_sole_match) {
    var layout, warning;
    switch (match.pattern) {
      case 'dictionary':
        return this.get_dictionary_match_feedback(match, is_sole_match);
      case 'spatial':
        layout = match.graph.toUpperCase();
        warning = match.turns === 1 ? '直线按键很容易猜到' : '短键盘模式很容易猜到';
        return {
          warning: warning,
          suggestions: ['使用更长的键盘图案和更多的转弯']
        };
      case 'repeat':
        warning = match.base_token.length === 1 ? '像“aaa”这样的重复很容易猜到' : '像“abcabcabc”这样的重复只比“abc”稍微难以猜测';
        return {
          warning: warning,
          suggestions: ['避免重复的单词和字符']
        };
      case 'sequence':
        return {
          warning: "像abc或6543这样的序列很容易猜到",
          suggestions: ['避免有序字符']
        };
      case 'regex':
        if (match.regex_name === 'recent_year') {
          return {
            warning: "当前年份很容易猜到",
            suggestions: ['避免当前年份', '避免与你相关的日期']
          };
        }
        break;
      case 'date':
        return {
          warning: "日期通常很容易猜到",
          suggestions: ['避免与您相关的日期和年份']
        };
    }
  },
  get_dictionary_match_feedback: function(match, is_sole_match) {
    var ref, result, suggestions, warning, word;
    warning = match.dictionary_name === 'passwords' ? is_sole_match && !match.l33t && !match.reversed ? match.rank <= 10 ? '这是十大常用密码' : match.rank <= 100 ? '这是前100个常用密码' : '这是一个非常常见的密码' : match.guesses_log10 <= 4 ? '这类似于常用的密码' : void 0 : match.dictionary_name === 'english_wikipedia' ? is_sole_match ? '单词本身很容易猜到' : void 0 : (ref = match.dictionary_name) === 'surnames' || ref === 'male_names' || ref === 'female_names' ? is_sole_match ? '姓名和姓氏本身很容易猜到' : '常见的名字和姓氏很容易猜到' : '';
    suggestions = [];
    word = match.token;
    if (word.match(scoring.START_UPPER)) {
      suggestions.push("大写化并没有多大帮助");
    } else if (word.match(scoring.ALL_UPPER) && word.toLowerCase() !== word) {
      suggestions.push("全大写几乎像全小写一样容易猜测");
    }
    if (match.reversed && match.token.length >= 4) {
      suggestions.push("反转单词并不难猜");
    }
    if (match.l33t) {
      suggestions.push("用‘@’替换a这样的可预测替换并没有多大帮助");
    }
    result = {
      warning: warning,
      suggestions: suggestions
    };
    return result;
  }
};

module.exports = feedback;

//# sourceMappingURL=feedback.js.map
